/* APPLE LOCAL  nested functions */
/* { dg-xfail-if "" { "*-*-darwin*" } { "*" } { "" } } */
/* PR c/7622 */

/* Verify that GCC can handle the mix of
   extern inline and nested functions. */

extern inline int t()
{
  int q() { return 0; }

  return q();
}

int foo()
{
  return t();
}
